#include "rpc_node.h"
#include "svc_rpc.h"
#include "ty_cJSON.h"
#include "tuya_ipc_rpc_auto_api.h"
#include "uni_log.h"

int tuya_ipc_rpc_server_declare_all()
{
    tuya_ipc_rpc_server_declare_sd();
    tuya_ipc_rpc_server_declare_api_ipc();
    tuya_ipc_rpc_server_declare_cloud_storage();
    tuya_ipc_rpc_server_declare_ptz();
    tuya_ipc_rpc_server_declare_webrtc();
    tuya_ipc_rpc_server_declare_gateway_stream();
    tuya_ipc_rpc_server_declare_ring_buffer();
    tuya_ipc_rpc_server_declare_streaming_p2p();
    tuya_ipc_rpc_server_declare_video_msg();
    return 0;
}

static int rpc_node_aes(uint8_t **data, uint32_t *len, uint8_t key[16], tuya_rpc_crypt_mode_t mode)
{
    if (mode == TUYA_RPC_ENCRYPT) {
        return aes128_ecb_encode(*data, *len, data, len, key);
    } else if (mode == TUYA_RPC_DECRYPT) {
        return aes128_ecb_decode(*data, *len, data, len, key);
    } else if (mode == TUYA_RPC_FREECRYPT) {
        return aes_free_data(*data);
    }

    return OPRT_COM_ERROR;
}

int tuya_ipc_rpc_server_func_call_cb(ty_cJSON *req_json, ty_cJSON **response)
{
    jrpc_msg_t request = {0}, reply= {0};

    if (NULL == req_json) {
        PR_ERR("create json err");
        return OPRT_CR_CJSON_ERR;
    }

    char* cmd = ty_cJSON_GetObjectItem(req_json, "cmd")->valuestring;
    PR_ERR("recv cmd: %s", cmd);
    jrpc_create_request(&request, cmd, 1);

    ty_cJSON* data_json = ty_cJSON_GetObjectItem(req_json, "data");
    if (data_json != NULL)
    {
        ty_cJSON *c =data_json->child;
        while (c)
        {
            if (ty_cJSON_IsNumber(c))
            {
                PR_ERR("int param: %s = %d", c->string, c->valueint);
                jrpc_write_int(&request, JRPC_PARAMS, c->string, c->valueint);
            }
            else if (ty_cJSON_IsString(c))
            {
                PR_ERR("str param: %s = %s", c->string, c->valuestring);
                jrpc_write_string(&request, JRPC_PARAMS, c->string, c->valuestring);
            }
            else
                PR_ERR("request type not support");

            c = c->next;
        }
    }
    else{
        PR_ERR("data is NULL");
    }

    int ret = tuya_ipc_rpc_server_call(cmd, &request, NULL, &reply);

    ty_cJSON* result_json = ty_cJSON_DetachItemFromObject(reply.json, "result");
    if (result_json == NULL){
        PR_ERR("result_json is NULL\n");
    }
    *response = result_json;

    jrpc_delete(&request);
    jrpc_delete(&reply);

    return ret;
}

