/**
 * @file tuya_ipc_rpc_server_gateway_stream.c
 * @brief  rpc ring_buffer adapter
 * @version 1.0
 * @date 2021-11-25
 *
 * @copyright Copyright 2021-2031 Tuya Inc. All Rights Reserved.
 *
 */


#include <stdio.h> 
#include "svc_rpc.h"
#include "tuya_error_code.h"
#include "tuya_ipc_streamer.h"
#include "uni_log.h"

void rpc_ipc_server_tuya_ipc_gateway_streamer_init(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
     
    if(OPRT_OK != (ret = tuya_ipc_gateway_streamer_init())) {
        PR_DEBUG("tuya_ipc_ring_buffer_init err \n");
        goto exit ;
    }
    
exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}


void rpc_ipc_server_tuya_ipc_gateway_stream_alive_cnt(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = tuya_ipc_gateway_stream_alive_cnt();    

    PR_DEBUG("gateway_stream_alive_cnt :%d \n",ret);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}

void rpc_ipc_server_tuya_ipc_gateway_stream_pause(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
     
    if(OPRT_OK != (ret = tuya_ipc_gateway_stream_pause())) {
        PR_DEBUG("tuya_ipc_ring_buffer_init err \n");
        goto exit ;
    }
    
exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}

void rpc_ipc_server_tuya_ipc_gateway_stream_resume(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
     
    if(OPRT_OK != (ret = tuya_ipc_gateway_stream_resume())) {
        PR_DEBUG("tuya_ipc_ring_buffer_init err \n");
        goto exit ;
    }
    
exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}

void rpc_ipc_server_tuya_ipc_gateway_stream_stop(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
     
    if(OPRT_OK != (ret = tuya_ipc_gateway_stream_stop())) {
        PR_DEBUG("tuya_ipc_ring_buffer_init err \n");
        goto exit ;
    }
    
exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}

void rpc_ipc_server_tuya_ipc_gateway_stream_set_prefer_mode(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    STREAM_TRANS_MODE_E transMode = 0 ;    
        
    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"transMode",&transMode))) {
        PR_DEBUG("device err \n");
        goto exit ;
    }


    if (OPRT_OK != ( ret = tuya_ipc_gateway_stream_set_prefer_mode(transMode))) {
        PR_DEBUG("tuya_ipc_gateway_stream_set_prefer_mode err \n");
        goto exit ;
    }
    
    PR_DEBUG("transMode :%d \n",transMode);

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}

void tuya_ipc_rpc_server_declare_gateway_stream()
{
    tuya_ipc_rpc_server_declare("tuya_ipc_gateway_streamer_init", rpc_ipc_server_tuya_ipc_gateway_streamer_init, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_gateway_stream_alive_cnt", rpc_ipc_server_tuya_ipc_gateway_stream_alive_cnt, NULL);
    
    tuya_ipc_rpc_server_declare("tuya_ipc_gateway_stream_pause", rpc_ipc_server_tuya_ipc_gateway_stream_pause, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_gateway_stream_resume", rpc_ipc_server_tuya_ipc_gateway_stream_resume, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_gateway_stream_stop", rpc_ipc_server_tuya_ipc_gateway_stream_stop, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_gateway_stream_set_prefer_mode", rpc_ipc_server_tuya_ipc_gateway_stream_set_prefer_mode, NULL); 
}


