/**
 * @file tuya_ipc_rpc_server_webrtc.c
 * @brief  rpc webrtc adapter
 * @version 1.0
 * @date 2021-11-25
 *
 * @copyright Copyright 2021-2031 Tuya Inc. All Rights Reserved.
 *
 */

#include <stdio.h> 
#include "svc_rpc.h"
#include "tuya_error_code.h"
#include "tuya_ipc_streamer.h"
#include "uni_log.h"

void rpc_ipc_server_tuya_ipc_webrtc_init(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
     
    if(OPRT_OK != (ret = tuya_ipc_webrtc_init())) {
        PR_DEBUG("tuya_ipc_webrtc_init err \n");
        goto exit ;
    }
    
exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}


void rpc_ipc_server_tuya_ipc_webrtc_deinit(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    if(OPRT_OK != (ret = tuya_ipc_webrtc_deinit())) {
        PR_DEBUG("rpc_ipc_server_tuya_ipc_webrtc_deinit err \n");
        goto exit ;
    }
    
exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}


void rpc_ipc_server_tuya_ipc_webrtc_stop(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
     
    if(OPRT_OK != (ret = tuya_ipc_webrtc_stop())) {
        PR_DEBUG("rpc_ipc_server_tuya_ipc_webrtc_stop err \n");
        goto exit ;
    }
    
exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}

void rpc_ipc_server_tuya_ipc_webrtc_alive_cnt(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T   webrtcalive_cnt = 0 ;
    INT_T ret = OPRT_OK;
     if(OPRT_OK != (ret = tuya_ipc_webrtc_alive_cnt(&webrtcalive_cnt)) ) {
        PR_DEBUG("tuya_ipc_webrtc_alive_cnt err \n");
        goto exit ;
    }
   PR_DEBUG("webrtcalive_cnt :%d \n",webrtcalive_cnt);
     
exit:

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    if(OPRT_OK == ret) {
    jrpc_write_int(reply, JRPC_RESULT, "webrtcalive_cnt", webrtcalive_cnt);
    }
}

void rpc_ipc_server_tuya_ipc_webRt_pause_set(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    CHAR_T pause_flag = 0 ;    
        
    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"pause_flag",&pause_flag))) {
        PR_DEBUG("pause_flag err \n");
        goto exit ;
    }

    PR_DEBUG("pause_flag:%d \n",pause_flag);

    if (OPRT_OK != ( ret = tuya_ipc_webRt_pause_set(pause_flag))) {
        PR_DEBUG("tuya_ipc_webRt_pause_set err \n");
        goto exit ;
    }

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}

void tuya_ipc_rpc_server_declare_webrtc()
{
    tuya_ipc_rpc_server_declare("tuya_ipc_webrtc_init", rpc_ipc_server_tuya_ipc_webrtc_init, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_webrtc_deinit", rpc_ipc_server_tuya_ipc_webrtc_deinit, NULL);
    
    tuya_ipc_rpc_server_declare("tuya_ipc_webrtc_stop", rpc_ipc_server_tuya_ipc_webrtc_stop, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_webrtc_alive_cnt", rpc_ipc_server_tuya_ipc_webrtc_alive_cnt, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_webRt_pause_set", rpc_ipc_server_tuya_ipc_webRt_pause_set, NULL);
}


