/*********************************************************************************
  *Copyright(C),2015-2020, 
  *TUYA 
  *www.tuya.comm
  *FileName:    tuya_ipc_media_demo
**********************************************************************************/
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/statfs.h>  
#include "tuya_ipc_common_demo.h"
#include "tuya_ipc_api.h"
#include "tuya_ring_buffer.h"
#include "tuya_ipc_media_demo.h"
#include "tuya_ipc_media.h"
#include "tuya_ipc_log_demo.h"
#include "tuya_ipc_media_update_demo.h"

#define AUDIO_FRAME_SIZE 640
#define AUDIO_FPS 25
#define VIDEO_BUF_SIZE	(1024 * 400) //Maximum frame

extern CHAR_T s_raw_path[128];
RING_BUFFER_USER_HANDLE_T g_a_handle = NULL ;
RING_BUFFER_USER_HANDLE_T g_v_handle = NULL ;
RING_BUFFER_USER_HANDLE_T g_v_handle2 = NULL ;
STATIC BOOL_T s_ring_buffer_inited = FALSE;

extern int g_meida_update_type ;
extern int g_media_update_wait ;

int IPC_APP_get_snapshot(char *snap_addr, int *snap_size)
{
    if (snap_addr == NULL || snap_size == NULL)
    {
        PR_ERR("get snapshot failed, param error\n");
        return -1;
    }

    //we use file to simulate
    char snapfile[128] = {0};
    *snap_size = 0;
    extern char s_raw_path[];
    PR_INFO("get snapshot\n");
    snprintf(snapfile, sizeof(snapfile), "%s/resource/media/demo_snapshot.jpg",s_raw_path);
    FILE*fp = fopen(snapfile,"r+");
    if(NULL == fp)
    {
        PR_ERR("fail to open snap.jpg\n");
        return -1;
    }
    fseek(fp,0,SEEK_END);
    *snap_size = ftell(fp);
    if(*snap_size < 100*1024)
    {
        fseek(fp,0,SEEK_SET);
        fread(snap_addr,*snap_size,1,fp);
    }
    fclose(fp);
    return 0;
}

/* This is for demo only. Should be replace with real PCM/AAC/G711 encoder output */
void *thread_live_audio(void *arg)
{
    char fullpath[128] = {0};
    sprintf(fullpath, "%s/resource/media/demo_audio.raw", s_raw_path);

    FILE *aFp = fopen(fullpath, "rb");
    if(aFp == NULL)
    {
        PR_ERR("can't read live audio file %s\n",fullpath);
        pthread_exit(0);
    }
    char audioBuf[AUDIO_FRAME_SIZE];
    MEDIA_FRAME_T pcm_frame = {0};
    pcm_frame.type = E_AUDIO_FRAME;

    while(1)
    {
        if( g_media_update_wait == 1) {
            usleep(100*1000);
            continue ;
        }

        if(g_a_handle == NULL) {
        	g_a_handle = tuya_ipc_ring_buffer_open(0, 0, E_IPC_STREAM_AUDIO_MAIN, E_RBUF_WRITE);
            if (g_a_handle == NULL) {
                PR_ERR("tuya_ipc_ring_buffer_open err  ");
                sleep(1);
                continue;
            }
            PR_DEBUG("tuya_ipc_ring_buffer_open success  ");
        }
        int size = fread(audioBuf, 1, AUDIO_FRAME_SIZE, aFp);
        if(size < AUDIO_FRAME_SIZE)
        {
            rewind(aFp);
            continue;
        }
        int frameRate = AUDIO_FPS;
        int sleepTick = 1000000/frameRate;
        static UINT64_T pts = 0;
        pts += sleepTick;
        pcm_frame.size = size;
        pcm_frame.p_buf = audioBuf;
        pcm_frame.pts = pts;
        TUYA_APP_Put_Frame(g_a_handle,&pcm_frame);

        usleep(sleepTick);
    }
    if(g_a_handle) {
	    tuya_ipc_ring_buffer_close(g_a_handle);   
	    g_a_handle = NULL ;
    }
    pthread_exit(0);
}

/* This is for demo only. Should be replace with real H264 encoder output */
int read_one_frame_from_demo_video_file(unsigned char *pVideoBuf,unsigned int offset,unsigned int BufSize,unsigned int *IskeyFrame,unsigned int 
                                        *FramLen,unsigned int *Frame_start)
{
    int pos = 0;
    int bNeedCal = 0;
    unsigned char NalType=0;
    int idx=0;
    if(BufSize<=5)
    {
        PR_ERR("bufSize is too small\n");
        return -1;
    }
    for(pos=0;pos <= BufSize-5;pos++)
    {
        if(pVideoBuf[pos]==0x00
            &&pVideoBuf[pos+1]==0x00
            &&pVideoBuf[pos+2]==0x00
            &&pVideoBuf[pos+3]==0x01)
        {
            NalType = pVideoBuf[pos + 4] & 0x1f;
            if(NalType == 0x7)
            {
                if(bNeedCal==1)
                {
                    *FramLen=pos-idx;
                    return 0;
                }
                
                *IskeyFrame = 1;
                *Frame_start = offset+pos;
                bNeedCal=1;
                idx=pos;
            }
            else if(NalType ==0x1)                
            {
               if(bNeedCal)
               {
                  *FramLen=pos-idx;
                  return 0;
               }
               *Frame_start=offset+pos;
               *IskeyFrame = 0;
               idx = pos;
               bNeedCal=1;
            }
        }
    }

    *FramLen=BufSize;
    
    return 0;
}

int read_one_frame_from_demo_video_file_hevc(unsigned char *pVideoBuf,unsigned int offset,unsigned int BufSize,unsigned int *IskeyFrame,unsigned int
*FramLen,unsigned int *Frame_start)
{
    int pos = 0;
    int bNeedCal = 0;
    unsigned char NalType=0;
    int idx=0;
    if(BufSize<=5)
    {
        printf("bufSize is too small\n");
        return -1;
    }
    for(pos=0;pos <= BufSize-5;pos++)
    {
        if(pVideoBuf[pos]==0x00
            &&pVideoBuf[pos+1]==0x00
            &&pVideoBuf[pos+2]==0x00
            &&pVideoBuf[pos+3]==0x01)
        {
            NalType = (pVideoBuf[pos + 4] & 0x7E) >> 1;
            if(NalType == 0x20)
            {
                if(bNeedCal==1)
                {
                    *FramLen=pos-idx;
                    return 0;
                }

                *IskeyFrame = 1;
                *Frame_start = offset+pos;
                bNeedCal=1;
                idx=pos;
            }
            else if(NalType ==0x1)
            {
               if(bNeedCal)
               {
                  *FramLen=pos-idx;
                  return 0;
               }
               *Frame_start=offset+pos;
               *IskeyFrame = 0;
               idx = pos;
               bNeedCal=1;
            }
        }
    }

    *FramLen = BufSize;
    return 0;
}

void *thread_live_video(void *arg)
{
    char raw_fullpath[128] = {0};
    char info_fullpath[128] = {0};
    unsigned int FrameLen=0,Frame_start=0;
    unsigned int offset=0;
    unsigned int IsKeyFrame=0;
    unsigned char *pVideoBuf = NULL;
    sprintf(raw_fullpath, "%s/resource/media/demo_video.264", s_raw_path);

    PR_DEBUG("start live video using %s",raw_fullpath);

    FILE *streamBin_fp = fopen(raw_fullpath, "rb");
    if((streamBin_fp == NULL))
    {
        PR_ERR("can't read live video file %s\n",raw_fullpath);
        pthread_exit(0);
    }
    fseek(streamBin_fp, 0, SEEK_END);
    UINT_T file_size = ftell(streamBin_fp);
    fseek(streamBin_fp, 0, SEEK_SET);
    pVideoBuf = malloc(file_size);
    fread(pVideoBuf, 1, file_size, streamBin_fp);

    char raw_fullpath_hevc[128] = { 0 };
    sprintf(raw_fullpath_hevc, "%s/resource/media/demo_video.265", s_raw_path);
    PR_DEBUG("start live video using %s",raw_fullpath_hevc);
    FILE *streamBin_fp_hevc = fopen(raw_fullpath_hevc, "rb");
    if ((streamBin_fp_hevc == NULL)) {
        PR_DEBUG("can't read live video file %s\n", raw_fullpath_hevc);
        pthread_exit(0);
    }
    fseek(streamBin_fp_hevc, 0, SEEK_END);
    UINT_T file_size_hevc = ftell(streamBin_fp_hevc);
    fseek(streamBin_fp_hevc, 0, SEEK_SET);
    unsigned char *pVideoBuf_hevc  = malloc(file_size_hevc);
    fread(pVideoBuf_hevc, 1, file_size_hevc, streamBin_fp_hevc);
    
    MEDIA_FRAME_T video_frame = {0};
    
    while(1)
    {
        if( g_media_update_wait == 1) {
            offset = 0 ;
            Frame_start = 0 ;
            FrameLen = 0 ;
            IsKeyFrame = 0 ;
            usleep(100*1000);
            continue ;
        }

		if(g_v_handle == NULL) {
			g_v_handle = tuya_ipc_ring_buffer_open(0, 0, E_IPC_STREAM_VIDEO_MAIN, E_RBUF_WRITE);
		    if (g_v_handle == NULL) {
		        PR_ERR("tuya_ipc_ring_buffer_open err  ");
		        sleep(1);
		        continue;
		    }
		    PR_DEBUG("tuya_ipc_ring_buffer_open success  ");
		}
		if(g_v_handle2 == NULL) {
		    g_v_handle2 = tuya_ipc_ring_buffer_open(0, 0, E_IPC_STREAM_VIDEO_SUB, E_RBUF_WRITE);
		    if (g_v_handle2 == NULL) {
		         PR_ERR("tuya_ipc_ring_buffer_open err  ");
		        sleep(1);
		        continue;
		    }
		    PR_DEBUG("tuya_ipc_ring_buffer_open success  ");
		}

        if(g_meida_update_type != MEDIA_UPDATE_H265) {

            offset=Frame_start+FrameLen;
            if(offset>=file_size)
            {
                offset=0;
            }
            read_one_frame_from_demo_video_file(pVideoBuf+offset,offset,file_size-offset,&IsKeyFrame,&FrameLen,&Frame_start);
            //Note: For I frame of H264, SPS/PPS/SEI/IDR should be combined within one frame, and the NALU separator should NOT be deleted.
            if(IsKeyFrame==1)
            {
                video_frame.type = E_VIDEO_I_FRAME;
                video_frame.size = FrameLen;
            }
            else
            {
                video_frame.type = E_VIDEO_PB_FRAME;
                video_frame.size = FrameLen;
            }

            int frameRate = 30;
            int sleepTick = 1000000/frameRate;
            static UINT64_T pts = 0;
            pts += sleepTick;

            video_frame.p_buf = pVideoBuf+Frame_start;
            video_frame.pts = pts;
            /* Send HD video data to the SDK */
            TUYA_APP_Put_Frame(g_v_handle, &video_frame);
            /* Send SD video data to the SDK */
            TUYA_APP_Put_Frame(g_v_handle2, &video_frame);
            usleep(sleepTick);
            } else  {

                offset = Frame_start + FrameLen;
                if (offset >= file_size_hevc) {
                    offset = 0;
                }
                read_one_frame_from_demo_video_file_hevc(pVideoBuf_hevc + offset, offset, file_size_hevc - offset, &IsKeyFrame, &FrameLen, &Frame_start);
                //Note: For I frame of H264, SPS/PPS/SEI/IDR should be combined within one frame, and the NALU separator should NOT be deleted.
                if (IsKeyFrame == 1) {
                    video_frame.type = E_VIDEO_I_FRAME;
                    video_frame.size = FrameLen;
                } else {
                    video_frame.type = E_VIDEO_PB_FRAME;
                    video_frame.size = FrameLen;
                }

                int frameRate = 25;        //30;
                int sleepTick = 1000000 / frameRate;
                static UINT64_T pts = 0;
                pts += sleepTick;

                video_frame.p_buf = pVideoBuf_hevc + Frame_start;
                video_frame.pts = pts;
                /* Send HD video data to the SDK */
                TUYA_APP_Put_Frame(g_v_handle, &video_frame);
                /* Send SD video data to the SDK */
                 TUYA_APP_Put_Frame(g_v_handle2, &video_frame);
                usleep(sleepTick);
        }
    }
    if(g_v_handle) {
        tuya_ipc_ring_buffer_close(g_v_handle);
        g_v_handle = NULL ;
    }

    if(g_v_handle2) {
        tuya_ipc_ring_buffer_close(g_v_handle2);
        g_v_handle2 = NULL ;
    }
    
    pthread_exit(0);
}

#if defined(IPC_CHANNEL_NUM) && (IPC_CHANNEL_NUM==2)
void *thread_live_video2(void *arg)
{
    char raw_fullpath[128] = {0};
    char info_fullpath[128] = {0};
    unsigned int FrameLen=0,Frame_start=0;
    unsigned int offset=0;
    unsigned int IsKeyFrame=0;
    unsigned char *pVideoBuf = NULL;
    sprintf(raw_fullpath, "%s/resource/media2/demo_video.264", s_raw_path);

    PR_DEBUG("start live video using %s",raw_fullpath);

	g_v_handle = tuya_ipc_ring_buffer_open(0, 1, 0, E_RBUF_WRITE);
	if(g_v_handle == NULL)
	{
		pthread_exit(0);
	}
    
	g_v_handle2 = tuya_ipc_ring_buffer_open(0, 1, 1, E_RBUF_WRITE);
	if(g_v_handle2 == NULL)
	{
		pthread_exit(0);
	}
    FILE *streamBin_fp = fopen(raw_fullpath, "rb");
    if((streamBin_fp == NULL))
    {
        PR_ERR("can't read live video file %s\n",raw_fullpath);
        pthread_exit(0);
    }
    fseek(streamBin_fp, 0, SEEK_END);
    UINT_T file_size = ftell(streamBin_fp);
    fseek(streamBin_fp, 0, SEEK_SET);
    pVideoBuf = malloc(file_size);
    fread(pVideoBuf, 1, file_size, streamBin_fp);

    MEDIA_FRAME_T h264_frame = {0};
    while(1)
    {
        offset=Frame_start+FrameLen;
        if(offset>=file_size)
        {
            offset=0;
        }
        read_one_frame_from_demo_video_file(pVideoBuf+offset,offset,file_size-offset,&IsKeyFrame,&FrameLen,&Frame_start);
        //Note: For I frame of H264, SPS/PPS/SEI/IDR should be combined within one frame, and the NALU separator should NOT be deleted.
        if(IsKeyFrame==1)
        {
            h264_frame.type = E_VIDEO_I_FRAME;
            h264_frame.size = FrameLen;
        }
        else
        {
            h264_frame.type = E_VIDEO_PB_FRAME;
            h264_frame.size = FrameLen;
        }

        int frameRate = 30;
        int sleepTick = 1000000/frameRate;
        static UINT64_T pts = 0;
        pts += sleepTick;

        h264_frame.p_buf = pVideoBuf+Frame_start;
        h264_frame.pts = pts;

        /* Send HD video data to the SDK */
        TUYA_APP_Put_Frame(v_handle, &h264_frame);
        /* Send SD video data to the SDK */
         TUYA_APP_Put_Frame(v_handle2, &h264_frame);

        usleep(sleepTick);
    }
	tuya_ipc_ring_buffer_close(g_v_handle2);
    g_v_handle2 = NULL ;
    pthread_exit(0);
}
#endif


VOID IPC_APP_Init_Media_Task()
{
#ifdef __HuaweiLite__
    TSK_INIT_PARAM_S stappTask;
    int taskid = -1;
    memset(&stappTask, 0, sizeof(TSK_INIT_PARAM_S));
    stappTask.pfnTaskEntry = (TSK_ENTRY_FUNC)thread_live_video;
    stappTask.uwStackSize  = 0x80000;
    stappTask.pcName = "live_video";
    stappTask.usTaskPrio = 10;
    stappTask.uwResved   = LOS_TASK_STATUS_DETACHED;
    LOS_TaskCreate((UINT32 *)&taskid, &stappTask);

    stappTask.pfnTaskEntry = (TSK_ENTRY_FUNC)thread_live_audio;
    stappTask.pcName = "live_video";
    LOS_TaskCreate((UINT32 *)&taskid, &stappTask);
#else
    pthread_t h264_output_thread;
    pthread_create(&h264_output_thread, NULL, thread_live_video, NULL);
    pthread_detach(h264_output_thread);

#if defined(IPC_CHANNEL_NUM) && (IPC_CHANNEL_NUM==2)
    pthread_t h264_output_thread2;
    pthread_create(&h264_output_thread2, NULL, thread_live_video2, NULL);
    pthread_detach(h264_output_thread2);
#endif

    pthread_t pcm_output_thread;
    pthread_create(&pcm_output_thread, NULL, thread_live_audio, NULL);
    pthread_detach(pcm_output_thread);
#endif    

    return;
}

/*
---------------------------------------------------------------------------------
code related RingBuffer
---------------------------------------------------------------------------------
*/
IPC_MEDIA_INFO_T old_mediaInfo = { 0 } ;
OPERATE_RET TUYA_APP_Init_Ring_Buffer(CONST IPC_MEDIA_INFO_T *pMediaInfo)
{
	if(NULL == pMediaInfo)
	{
		PR_ERR("create ring buffer para is NULL\n");
		return -1;
	}
	OPERATE_RET ret = OPRT_OK;

    if(s_ring_buffer_inited == TRUE)
    {
        PR_DEBUG("The Ring Buffer Is Already Inited");
        return OPRT_OK;
    }

    IPC_STREAM_E ringbuffer_stream_type;
   // CHANNEL_E channel;
    RING_BUFFER_INIT_PARAM_T param={0};
    for( ringbuffer_stream_type = E_IPC_STREAM_VIDEO_MAIN; ringbuffer_stream_type < E_IPC_STREAM_MAX; ringbuffer_stream_type++ )
    {
        PR_DEBUG("init ring buffer Channel:%d Enable:%d", ringbuffer_stream_type, pMediaInfo->stream_enable[ringbuffer_stream_type]);
        if(pMediaInfo->stream_enable[ringbuffer_stream_type] == TRUE)
        {
            if(ringbuffer_stream_type == E_IPC_STREAM_AUDIO_MAIN)
            {
                param.bitrate = pMediaInfo->audio_sample[E_IPC_STREAM_AUDIO_MAIN]*pMediaInfo->audio_databits[E_IPC_STREAM_AUDIO_MAIN]/1024;
                param.fps = pMediaInfo->audio_fps[E_IPC_STREAM_AUDIO_MAIN];
                param.max_buffer_seconds = 0;
                param.request_key_frame_cb = NULL;
                PR_DEBUG("audio_sample %d, audio_databits %d, audio_fps %d",pMediaInfo->audio_sample[E_IPC_STREAM_AUDIO_MAIN],pMediaInfo->audio_databits[E_IPC_STREAM_AUDIO_MAIN],pMediaInfo->audio_fps[E_IPC_STREAM_AUDIO_MAIN]);
                ret = tuya_ipc_ring_buffer_init(0, 0, ringbuffer_stream_type,&param);
            }
            else
            {
                param.bitrate = pMediaInfo->video_bitrate[ringbuffer_stream_type];
                param.fps = pMediaInfo->video_fps[ringbuffer_stream_type];
                param.max_buffer_seconds = 0;
                param.request_key_frame_cb = NULL;
                PR_DEBUG("video_bitrate %d, video_fps %d",pMediaInfo->video_bitrate[ringbuffer_stream_type],pMediaInfo->video_fps[ringbuffer_stream_type]);
                ret = tuya_ipc_ring_buffer_init(0, 0, ringbuffer_stream_type, &param);
            }
            if(ret != 0)
            {
                PR_ERR("init ring buffer fails. %d %d", ringbuffer_stream_type, ret);
                return OPRT_MALLOC_FAILED;
            }
            PR_DEBUG("init ring buffer success. channel:%d", ringbuffer_stream_type);
        }
    }

     memcpy(&old_mediaInfo,pMediaInfo,sizeof(IPC_MEDIA_INFO_T)) ;
    s_ring_buffer_inited = TRUE;

    return OPRT_OK;
}


OPERATE_RET TUYA_APP_DeInit_Ring_Buffer()
{
    PR_DEBUG("TUYA_APP_DeInit_Ring_Buffer START");
	OPERATE_RET ret = OPRT_OK;
    if(s_ring_buffer_inited == FALSE)
    {
        PR_DEBUG("The Ring Buffer has not Inited");
        return OPRT_OK;
    }
    //关闭所有ringbuffer的句柄
    if(g_v_handle) {
        tuya_ipc_ring_buffer_close(g_v_handle);
        g_v_handle = NULL ;
    }
    
    if(g_v_handle2) {
        tuya_ipc_ring_buffer_close(g_v_handle2); 
        g_v_handle2 = NULL  ;
    }
    
    if(g_a_handle) {
        tuya_ipc_ring_buffer_close(g_a_handle);
        g_a_handle = NULL ;
    }
     
    IPC_STREAM_E ringbuffer_stream_type;
    for( ringbuffer_stream_type = E_IPC_STREAM_VIDEO_MAIN; ringbuffer_stream_type < E_IPC_STREAM_MAX; ringbuffer_stream_type++ )
    {
        if(old_mediaInfo.stream_enable[ringbuffer_stream_type] == TRUE) {
            ret = tuya_ipc_ring_buffer_uninit(0, 0, ringbuffer_stream_type);
            if(ret != 0)
            {
                PR_ERR("tuya_ipc_ring_buffer_uninit fails. %d %d", ringbuffer_stream_type, ret);
                continue ;
                //return OPRT_MALLOC_FAILED;
            }
            PR_DEBUG("tuya_ipc_ring_buffer_uninit success. channel:%d", ringbuffer_stream_type);
        }
    }

    s_ring_buffer_inited = FALSE;

    return OPRT_OK;
}


OPERATE_RET TUYA_APP_Put_Frame(RING_BUFFER_USER_HANDLE_T handle, IN CONST MEDIA_FRAME_T *p_frame)
{
    PR_TRACE("Put Frame. type:%d size:%u pts:%llu ts:%llu",
             p_frame->type, p_frame->size, p_frame->pts, p_frame->timestamp);

    OPERATE_RET ret = tuya_ipc_ring_buffer_append_data(handle,p_frame->p_buf, p_frame->size,p_frame->type,p_frame->pts);

    if(ret != OPRT_OK)
    {
        PR_ERR("Put Frame Fail.%d  type:%d size:%u pts:%llu ts:%llu",ret,
                  p_frame->type, p_frame->size, p_frame->pts, p_frame->timestamp);
    }
    return ret;
}

OPERATE_RET TUYA_APP_get_snapshot_cb(IN INT_T device, IN INT_T channel, OUT CHAR_T *snap_addr, OUT INT_T *snap_size)
{
    // in this demo ignore device and channel parameter
    return IPC_APP_get_snapshot(snap_addr, snap_size);
}