#include <sys/time.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include "rtsp.h"
#include "rtsp_server.h"
#include "rtsp_client.h"
#include "tuya_ipc_log_demo.h"

static unsigned char video_buf[8 * 1024 * 1024];
static int video_len;
static char audio_buf[2 * 1024 * 1024];
static int audio_len;
static uint64_t time_now;

typedef struct {
    int pos[RTSP_MEDIA_TYPE_NUM];
    uint64_t last_pts[RTSP_MEDIA_TYPE_NUM];
} TUYA_RTSP_USER_INFO_T;

TUYA_RTSP_USER_INFO_T* users[5] = {0};

static uint64_t get_now_time()
{
    struct timeval tv = {0};
    gettimeofday(&tv, NULL);

    return tv.tv_sec * 1000LL + tv.tv_usec /1000;
}

static int get_sample_rate(RTSP_MEDIA_TYPE_E type)
{
    switch (type){
    case RTSP_MEDIA_TYPE_VIDEO:
        return 90000;
    case RTSP_MEDIA_TYPE_AUDIO:
        return 8000;
    default:
        return 8000;
    }
}
static int get_frame_264(int user_id, RTSP_MEDIA_TYPE_E type, char** buf, int *plen, uint64_t *pts)
{
    TUYA_RTSP_USER_INFO_T* pinfo = users[user_id];
    int pos = pinfo->pos[type];
    int start = -1, end = -1;
    int media_len = type == RTSP_MEDIA_TYPE_VIDEO ? video_len : audio_len;
    int frame_rate[2] = {28, 25};

    if (time_now - pinfo->last_pts[type] < 1000 / frame_rate[type]) {
        return -1;
    }

    if (type == RTSP_MEDIA_TYPE_VIDEO){

        for(; pos < media_len - 5; pos++){
            if (video_buf[pos] == 0 && video_buf[pos + 1] == 0
                && video_buf[pos + 2] == 0 && video_buf[pos + 3] == 1){
                if (-1 == start){
                    start = pos;
                }
                else {
                    int nal_pos = pos + 4;
                    int type = video_buf[nal_pos] & 0x1f;
                    if (type == 0x7 || type == 0x8){
                        continue;
                    }
                    end = pos;
                    break;
                }
            }
        }

        if (start != -1 && end == -1){
            end = media_len + 1;
        }

        *buf = (char*)video_buf + start;
    }
    else {
        *buf = audio_buf + pos;
        start = pos;
        end = pos + 320;
        if (end > audio_len){
            end = audio_len;
        }
    }

    if (pinfo->last_pts[type] == 0){
        *pts = time_now;
    }
    else{
        *pts = pinfo->last_pts[type] + 1000 / frame_rate[type];
    }

    *plen = end - start;

    pinfo->last_pts[type] = *pts;
    pinfo->pos[type] = end;

    if (pinfo->pos[type] > media_len - 5){
        pinfo->pos[type] = 0;
    }

    return 0;
}

static int get_frame_fragment_jpeg(int user_id, RTSP_FRAME_FRAGMENT_T* pframe)
{
    TUYA_RTSP_USER_INFO_T* pinfo = users[user_id];
    RTSP_FRAME_TYPE_E type = pframe->type;
    int pos = pinfo->pos[type];
    int start = -1, end = -1;
    int media_len = type == RTSP_MEDIA_TYPE_VIDEO ? video_len : audio_len;
    int frame_rate[2] = {10000, 10000};

    if (time_now - pinfo->last_pts[type] < 1000 / frame_rate[type]) {
        return -1;
    }

    if (type == RTSP_MEDIA_TYPE_VIDEO){

        start = pos;
        end = pos + 1024;
        if (end > media_len)
        {
            end = media_len-5;
        }

        pframe->buf = (char*)video_buf + start;
    }
    else {
        pframe->buf = audio_buf + pos;
        start = pos;
        end = pos + 640;
        if (end > audio_len){
            end = audio_len;
        }
    }
    pframe->len = end - start;
    pframe->pts = time_now;
    pframe->is_eof = 1;

    pinfo->last_pts[type] = time_now;
    pinfo->pos[type] = end;

    if (pinfo->pos[type] >= media_len-5){
        pinfo->pos[type] = 0;
    }

    return 0;
}


static int get_frame_jpeg(int user_id, RTSP_MEDIA_TYPE_E type, char** buf, int *plen, uint64_t *pts)
{
    TUYA_RTSP_USER_INFO_T* pinfo = users[user_id];
    int pos = pinfo->pos[type];
    int start = -1, end = -1;
    int media_len = type == RTSP_MEDIA_TYPE_VIDEO ? video_len : audio_len;
    int frame_rate[2] = {1, 25};

    if (time_now - pinfo->last_pts[type] < 1000 / frame_rate[type]) {
        return -1;
    }

    if (type == RTSP_MEDIA_TYPE_VIDEO){

        for(; pos < media_len - 1; pos++){
            if (video_buf[pos] == 0xFF && video_buf[pos + 1] == 0xD8 ){
                if (-1 == start){
                    start = pos;
                }
            }
            if (video_buf[pos] == 0xFF && video_buf[pos + 1] == 0xD9 ){
                if (start >= 0){
                    end = pos + 2;
                    break;
                }
            }
        }

        *buf = (char*)video_buf + start;
    }
    else {
        *buf = audio_buf + pos;
        start = pos;
        end = pos + 640;
        if (end > audio_len){
            end = audio_len;
        }
    }
    *plen = end - start;
    *pts = time_now;

    pinfo->last_pts[type] = time_now;
    pinfo->pos[type] = end;

    if (pinfo->pos[type] > media_len - 20000){
        pinfo->pos[type] = 0;
    }

    return 0;
}

static int get_frame_265(int user_id, RTSP_MEDIA_TYPE_E type, char** buf, int *plen, uint64_t *pts)
{
    TUYA_RTSP_USER_INFO_T* pinfo = users[user_id];
    int pos = pinfo->pos[type];
    int start = -1, end = -1;
    int media_len = type == RTSP_MEDIA_TYPE_VIDEO ? video_len : audio_len;
    int frame_rate[2] = {28, 50};

    if (time_now - pinfo->last_pts[type] < 1000 / frame_rate[type]) {
        return -1;
    }

    if (type == RTSP_MEDIA_TYPE_VIDEO){

#if 0
        for(; pos < media_len - 5; pos++){
            if (video_buf[pos] == 0 && video_buf[pos + 1] == 0
                && video_buf[pos + 2] == 0 && video_buf[pos + 3] == 1){
                if (-1 == start){
                    start = pos;
                }
                else {
                    int nal_pos = pos + 4;
                    int type = (video_buf[nal_pos] >> 1) & 0x3f;
                    if (type == 32 || type == 33 || type == 34){
                        continue;
                    }
                    end = pos;
                    break;
                }
            }
        }

        if (start != -1 && end == -1){
            end = media_len + 1;
        }
        #endif

        start = 0;
        end = video_len;
        *buf = (char*)video_buf + start;
    }
    else {
        *buf = audio_buf + pos;
        start = pos;
        end = pos + 320;
        if (end > audio_len){
            end = audio_len;
        }
    }
    *plen = end - start;
    *pts = time_now;

    pinfo->last_pts[type] = time_now;
    pinfo->pos[type] = end;

    if (pinfo->pos[type] > media_len - 5){
        pinfo->pos[type] = 0;
    }

    return 0;
}

static RTP_CODEC_E get_codec(RTSP_MEDIA_TYPE_E type)
{
    switch (type){
    case RTSP_MEDIA_TYPE_VIDEO:
        return RTP_CODEC_H264;
    case RTSP_MEDIA_TYPE_AUDIO:
        return RTP_CODEC_G711U;
    default:
        return 0;
    }
}
static int start()
{
    int i;
    for(i = 0; i < 5; i++){
        if (NULL == users[i]){
            users[i] = malloc(sizeof(TUYA_RTSP_USER_INFO_T));
            users[i]->last_pts[RTSP_MEDIA_TYPE_VIDEO] = 0;
            users[i]->last_pts[RTSP_MEDIA_TYPE_AUDIO] = 0;
            users[i]->pos[RTSP_MEDIA_TYPE_VIDEO] = 0;
            users[i]->pos[RTSP_MEDIA_TYPE_AUDIO] = 0;
            return i;
        }
    }
    return -1;
}
static int stop(int user_id)
{
    if (users[user_id]){
        free(users[user_id]);
        users[user_id] = NULL;
    }
    return 0;
}

static int get_name(char* buf, int* buf_len)
{
    char name[] = "stream_0";
    memcpy(buf, name, sizeof(name));

    *buf_len = sizeof(name);

    return 0;
}

static int start_server()
{
    FILE* fp = fopen("/tmp/demo_video.264", "r");
    if (!fp){
        PR_ERR("open video file failed\n");
        return -1;
    }
    video_len = fread(video_buf, 1, sizeof(video_buf), fp);
    fclose(fp);
    PR_INFO("read %d bytes from video file\n", video_len);

    fp = fopen("/tmp/demo_audio.raw", "r");
    if (!fp){
        PR_ERR("open audio file failed\n");
        return -1;
    }
    audio_len = fread(audio_buf, 1, sizeof(audio_buf), fp);
    fclose(fp);
    PR_INFO("read %d bytes from audio file\n", audio_len);


    time_now = get_now_time();

    RTSP_STREAM_SRC_T src = {0};
    src.get_codec = get_codec;
    src.get_frame = get_frame_264;
    //src.get_frame_fragment = get_frame_fragment_jpeg;
    src.get_sample_rate = get_sample_rate;
    src.start = start;
    src.stop = stop;
    src.get_name = get_name;

    tuya_ipc_rtsp_server_start("", "");
    tuya_ipc_rtsp_server_register_stream_src(&src);
    char url[256] = {0};
    //int url_len = 256;
    //rtsp_server_get_url(id, url, &url_len);
    //rtsp_server_set_password("111111");
    PR_INFO("rtsp server started, visit by %s\n", url);

    return 0;
}

typedef struct {
    RTSP_CLIENT_HANDLE_T handle;
    int idx;
    FILE* vfp;
    FILE* afp;
} rtsp_handle_info_s;

#define RTSP_HANDLE_NUM 2
static rtsp_handle_info_s hdl_info[RTSP_HANDLE_NUM];

static int put_data_test(RTSP_CLIENT_HANDLE_T handle, RTSP_DATA_PACKET_T* pkt, void* priv)
{
    PR_DEBUG("type(%3d) ts(%15llu) codec(%3d) start(%3d) end(%3d) size(%8d)\n", pkt->type, pkt->timestamp, pkt->codec,
            pkt->frame_start, pkt->frame_end, pkt->size);
    
    rtsp_handle_info_s* pinfo = NULL;
    int i;
    for(i = 0; i < RTSP_HANDLE_NUM; i++){
        if (handle == hdl_info[i].handle){
            pinfo = &hdl_info[i];
        }
    }

    if (pinfo){
        if (pkt->type == E_RTSP_AUDIO_FRAME){
            if (!pinfo->afp){
                pinfo->afp = fopen("audio.dump", "w");
            }

            fwrite(pkt->buf, pkt->size, 1, pinfo->afp);
        }
        else {
            if (!pinfo->vfp){
                pinfo->vfp = fopen("video.dump", "w");
            }
            fwrite(pkt->buf, pkt->size, 1, pinfo->vfp);
        }
    }

    return 0;
}

static int start_client(char* url)
{
    tuya_ipc_rtsp_client_init();

    RTSP_CLIENT_PARAM_T param = {0};

    if (url == NULL){
        url = "rtsp://192.168.186.128:8554/stream_0";
    }

    strcpy(param.url, url);
    strcpy(param.username, "admin");
    strcpy(param.password, "Admin123");
    param.trans_mode = RTSP_TRANSPORT_MODE_TCP;
    param.put_data_cb = put_data_test;

    PR_INFO("start client\n");
    RTSP_CLIENT_HANDLE_T handle = tuya_ipc_rtsp_client_start(&param);
    hdl_info[0].handle = handle;
    hdl_info[0].idx = 0;
    return 0;
}

int rtsp_main(int rtsp_mode, char* url)
{
    if (rtsp_mode == 1){
        PR_INFO("start client with url: %s\n", url);
        start_client(url);
    }
    else{
        start_server();
    }

    while(1) {
        time_now = get_now_time();
        usleep(1000);
    }

    return 0;
}